// Priorities.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "Priorities.h"
#include "iterations.h"
#include "Colors.h"

/*****************************************************************************
*                               THREAD_PRIORITY_INFO
*****************************************************************************/
typedef struct {
   int priority;
   UINT id;
} THREAD_PRIORITY_INFO, *LPTHREAD_PRIORITY_INFO;

#define DECLARE_THREAD_INFO(x) static THREAD_PRIORITY_INFO info_##x = { x, IDS_##x};

DECLARE_THREAD_INFO(THREAD_PRIORITY_IDLE)
DECLARE_THREAD_INFO(THREAD_PRIORITY_LOWEST)
DECLARE_THREAD_INFO(THREAD_PRIORITY_BELOW_NORMAL)
DECLARE_THREAD_INFO(THREAD_PRIORITY_NORMAL)
DECLARE_THREAD_INFO(THREAD_PRIORITY_ABOVE_NORMAL)
DECLARE_THREAD_INFO(THREAD_PRIORITY_HIGHEST)
DECLARE_THREAD_INFO(THREAD_PRIORITY_TIME_CRITICAL)



/****************************************************************************
*                              threadPriorities
*
* This is used to preload the thread priorities dropdown list.  It encodes
* the information (see previous table) so the priority number is available
* and we can find out the string as well
****************************************************************************/

#define DEFAULT_THREAD_PRIORITY_SELECTION ((DWORD_PTR)&info_THREAD_PRIORITY_NORMAL)

#define IDDATA_THREAD_PRIORITY(x) { IDS_##x, (DWORD_PTR)&info_##x}

static const IDData threadPriorities[] = {
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_IDLE),
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_LOWEST),
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_BELOW_NORMAL),
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_NORMAL),
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_ABOVE_NORMAL),
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_HIGHEST),
   IDDATA_THREAD_PRIORITY(THREAD_PRIORITY_TIME_CRITICAL),
   {0, 0} // EOT
};
// CPriorities

IMPLEMENT_DYNAMIC(CPriorities, CIDCombo)
CPriorities::CPriorities()
{
}

CPriorities::~CPriorities()
{
}


BEGIN_MESSAGE_MAP(CPriorities, CIDCombo)
END_MESSAGE_MAP()



// CPriorities message handlers


/****************************************************************************
*                           CPriorities::Initialize
* Result: void
*       
* Effect: 
*       Initializes the control
****************************************************************************/

void CPriorities::Initialize()
    {
     CIDCombo::AddStrings(threadPriorities);
    } // CPriorities::Initialize

/****************************************************************************
*                             CPriorities::Select
* Inputs:
*       int priority: Desired priority to select
* Result: BOOL
*       TRUE if successful
*       FALSE if error (THREAD_PRIORITY_NORMAL is selected)
* Effect: 
*       Selects the desired priority
****************************************************************************/

BOOL CPriorities::Select(int priority)
    {
     for(int i = 0; i < GetCount(); i++)
        { /* search */
         LPTHREAD_PRIORITY_INFO info = (LPTHREAD_PRIORITY_INFO)GetItemDataPtr(i);
         if(info->priority == priority)
            { /* found it */
             SetCurSel(i);
             return TRUE;
            } /* found it */
        } /* search */
     // Note this critically depends on the fact that the control is actually initialized
     // and therefore this recursion will not be infinite
     Select(THREAD_PRIORITY_NORMAL);
     return FALSE;
    } // CPriorities::Select

/****************************************************************************
*                         CPriorities::GetCurPriority
* Result: int
*       Current priority
****************************************************************************/

int CPriorities::GetCurPriority()
    {
     LPTHREAD_PRIORITY_INFO info = (LPTHREAD_PRIORITY_INFO)CIDCombo::GetCurItem();
     if(info == NULL)
        return THREAD_PRIORITY_NORMAL;
     return info->priority;
    } // CPriorities::GetCurPriority

/****************************************************************************
*                        CPriorities::GetCurPriorityName
* Result: CString
*       The name of the priority object
****************************************************************************/

CString CPriorities::GetCurPriorityName()
    {
     LPTHREAD_PRIORITY_INFO info = (LPTHREAD_PRIORITY_INFO)CIDCombo::GetCurItem();
     if(info == NULL)
        return _T("");

     CString s;
     s.LoadString(info->id);
     return s;
    } // CPriorities::GetCurPriorityName

/****************************************************************************
*                            CPriorities::DrawItem
* Inputs:
*       LPDRAWITEMSTRUCT dis
* Result: void
*       
* Effect: 
*       Draws the item
****************************************************************************/

void CPriorities::DrawItem(LPDRAWITEMSTRUCT dis)
    {
     CDC * dc = CDC::FromHandle(dis->hDC);
     
     if(dis->itemID == -1)
        { /* no items */
         CBrush bg(::GetSysColor(COLOR_WINDOW));
         dc->FillRect(&dis->rcItem, &bg);
         if(dis->itemState & ODS_FOCUS)
            { /* selected */
             dc->DrawFocusRect(&dis->rcItem);
            } /* selected */
         return;
        } /* no items */

     int saved = dc->SaveDC();

     COLORREF txcolor;
     COLORREF bkcolor;

     if(dis->itemState & ODS_SELECTED)
        { /* selected */
         if(::GetFocus() == m_hWnd)
            { /* has focus */
             bkcolor = ::GetSysColor(COLOR_HIGHLIGHT);
             txcolor = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
            } /* has focus */
         else
            { /* no focus */
             bkcolor = ::GetSysColor(COLOR_BTNFACE);
             txcolor = ::GetSysColor(COLOR_BTNTEXT);
            } /* no focus */
        } /* selected */
     else
        { /* not selected */
        txcolor = ::GetSysColor(COLOR_WINDOWTEXT);
         bkcolor = ::GetSysColor(COLOR_WINDOW);
        } /* not selected */

     {
      CBrush bg(bkcolor);
      dc->FillRect(&dis->rcItem, &bg);
     }

     dc->SetTextColor(txcolor);
     dc->SetBkMode(TRANSPARENT);

     //****************************************************************
     // +---------+
     // |         |
     // |         |
     // |         |
     // |         |
     // |         |
     // +---------+
     //****************************************************************

     int h = GetItemHeight(dis->itemID);
     CRect block = dis->rcItem;
     block.left = dis->rcItem.left;
     block.right = block.left + h;  // square box
     block.top++;
     block.bottom--;
     
     
     LPTHREAD_PRIORITY_INFO info = (LPTHREAD_PRIORITY_INFO)dis->itemData;
     dc->FillSolidRect(&block, PriorityColors::GetBkColor(info->priority));

     CString s;
     GetLBText(dis->itemID, s);
     int gap = 2 * ::GetSystemMetrics(SM_CXEDGE);
     dc->TextOut(block.right + gap, dis->rcItem.top, s);

     if(dis->itemState & ODS_FOCUS)
        dc->DrawFocusRect(&dis->rcItem);

     dc->RestoreDC(saved);
    }
