// ProcessPriority.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "ProcessPriority.h"
#include "unavailable.h"

/*****************************************************************************
*                              PROCESS_PRIORITY_INFO
*****************************************************************************/
typedef struct {
   DWORD priority;
   int threadPriority[16];
   UINT id;
} PROCESS_PRIORITY_INFO, * LPPROCESS_PRIORITY_INFO;



/****************************************************************************
*                                PriorityInfo
*
*       This table is used to decide how to color the priorities array       
****************************************************************************/


#define DEFAULT_PROCESS_PRIORITY (&PriorityInfo[2]) // normal priority

static const PROCESS_PRIORITY_INFO
   PriorityInfo[] = {
   { IDLE_PRIORITY_CLASS,          { UNAVAILABLE,                    //  0
                                     THREAD_PRIORITY_IDLE,           //  1
                                     THREAD_PRIORITY_LOWEST,         //  2
                                     THREAD_PRIORITY_BELOW_NORMAL,   //  3
                                     THREAD_PRIORITY_NORMAL,         //  4
                                     THREAD_PRIORITY_ABOVE_NORMAL,   //  5
                                     THREAD_PRIORITY_HIGHEST,        //  6
                                     UNAVAILABLE,                    //  7
                                     UNAVAILABLE,                    //  8
                                     UNAVAILABLE,                    //  9
                                     UNAVAILABLE,                    // 10
                                     UNAVAILABLE,                    // 11
                                     UNAVAILABLE,                    // 12
                                     UNAVAILABLE,                    // 13
                                     UNAVAILABLE,                    // 14
                                     THREAD_PRIORITY_TIME_CRITICAL}, // 15
                                   IDS_IDLE_PRIORITY_CLASS
   },
   { BELOW_NORMAL_PRIORITY_CLASS,  { UNAVAILABLE,
                                     THREAD_PRIORITY_IDLE,           //  1
                                     UNAVAILABLE,                    //  2
                                     UNAVAILABLE,                    //  3
                                     THREAD_PRIORITY_LOWEST,         //  4
                                     THREAD_PRIORITY_BELOW_NORMAL,   //  5
                                     THREAD_PRIORITY_NORMAL,         //  6
                                     THREAD_PRIORITY_ABOVE_NORMAL,   //  7
                                     THREAD_PRIORITY_HIGHEST,        //  8
                                     UNAVAILABLE,                    //  9
                                     UNAVAILABLE,                    // 10
                                     UNAVAILABLE,                    // 11
                                     UNAVAILABLE,                    // 12
                                     UNAVAILABLE,                    // 13
                                     UNAVAILABLE,                    // 14
                                     THREAD_PRIORITY_TIME_CRITICAL}, // 15
                                   IDS_BELOW_NORMAL_PRIORITY_CLASS
   },
   { NORMAL_PRIORITY_CLASS,        { UNAVAILABLE,
                                     THREAD_PRIORITY_IDLE,           //  1
                                     UNAVAILABLE,                    //  2
                                     UNAVAILABLE,                    //  3
                                     UNAVAILABLE,                    //  4
                                     THREAD_PRIORITY_LOWEST,         //  5
                                     THREAD_PRIORITY_BELOW_NORMAL,   //  6
                                     THREAD_PRIORITY_NORMAL,         //  7
                                     THREAD_PRIORITY_ABOVE_NORMAL,   //  8
                                     THREAD_PRIORITY_HIGHEST,        //  9
                                     UNAVAILABLE,                    // 10
                                     UNAVAILABLE,                    // 11
                                     UNAVAILABLE,                    // 12
                                     UNAVAILABLE,                    // 13
                                     UNAVAILABLE,                    // 14
                                     THREAD_PRIORITY_TIME_CRITICAL}, // 15
                                   IDS_NORMAL_PRIORITY_CLASS
   },
   { ABOVE_NORMAL_PRIORITY_CLASS,  { UNAVAILABLE,
                                     THREAD_PRIORITY_IDLE,           //  1
                                     UNAVAILABLE,                    //  2
                                     UNAVAILABLE,                    //  3
                                     UNAVAILABLE,                    //  4
                                     UNAVAILABLE,                    //  5
                                     UNAVAILABLE,                    //  6
                                     UNAVAILABLE,                    //  7
                                     THREAD_PRIORITY_LOWEST,         //  8
                                     THREAD_PRIORITY_BELOW_NORMAL,   //  9
                                     THREAD_PRIORITY_NORMAL,         // 10
                                     THREAD_PRIORITY_ABOVE_NORMAL,   // 11
                                     THREAD_PRIORITY_HIGHEST,        // 12
                                     UNAVAILABLE,                    // 13
                                     UNAVAILABLE,                    // 14
                                     THREAD_PRIORITY_TIME_CRITICAL}, // 15
                                   IDS_ABOVE_NORMAL_PRIORITY_CLASS
   },
   { HIGH_PRIORITY_CLASS,          { UNAVAILABLE,
                                     THREAD_PRIORITY_IDLE,           //  1
                                     UNAVAILABLE,                    //  2
                                     UNAVAILABLE,                    //  3
                                     UNAVAILABLE,                    //  4
                                     UNAVAILABLE,                    //  5
                                     UNAVAILABLE,                    //  6
                                     UNAVAILABLE,                    //  7
                                     UNAVAILABLE,                    //  8
                                     UNAVAILABLE,                    //  9
                                     UNAVAILABLE,                    // 10
                                     THREAD_PRIORITY_LOWEST,         // 11
                                     THREAD_PRIORITY_BELOW_NORMAL,   // 12
                                     THREAD_PRIORITY_NORMAL,         // 13
                                     THREAD_PRIORITY_ABOVE_NORMAL,   // 14
                                   //THREAD_PRIORITY_HIGHEST,        // 15
                                     THREAD_PRIORITY_TIME_CRITICAL}, // 15
                                   IDS_HIGH_PRIORITY_CLASS
   }
};
   

/****************************************************************************
*                              ProcessPriorities
*
* This is used to initialize the Proces Priorities dropdown lists
****************************************************************************/

static const IDData ProcessPriorities[] ={
   { IDS_IDLE_PRIORITY_CLASS,          (DWORD_PTR)&PriorityInfo[0]},
   { IDS_BELOW_NORMAL_PRIORITY_CLASS,  (DWORD_PTR)&PriorityInfo[1]},
   { IDS_NORMAL_PRIORITY_CLASS,        (DWORD_PTR)&PriorityInfo[2]},
   { IDS_ABOVE_NORMAL_PRIORITY_CLASS,  (DWORD_PTR)&PriorityInfo[3]},
   { IDS_HIGH_PRIORITY_CLASS,          (DWORD_PTR)&PriorityInfo[4]},
   { 0, 0 } // EOT
};


// CProcessPriority

IMPLEMENT_DYNAMIC(CProcessPriority, CIDCombo)
CProcessPriority::CProcessPriority()
{
}

CProcessPriority::~CProcessPriority()
{
}


BEGIN_MESSAGE_MAP(CProcessPriority, CIDCombo)
END_MESSAGE_MAP()



// CProcessPriority message handlers


/****************************************************************************
*                        CProcessPriority::Initialize
* Result: void
*       
* Effect: 
*       Initializes the control
****************************************************************************/

void CProcessPriority::Initialize()
    {
     AddStrings(ProcessPriorities);     
    } // CProcessPriority::Initialize

/****************************************************************************
*                      CProcessPriority::GetCurPriority
* Result: DWORD
*       The current selected process priority class
****************************************************************************/

DWORD CProcessPriority::GetCurPriority()
    {
     LPPROCESS_PRIORITY_INFO pinfo = (LPPROCESS_PRIORITY_INFO)GetCurItem();     
     if(pinfo == NULL)
        return NORMAL_PRIORITY_CLASS;
     else
        return pinfo->priority;
    } // CProcessPriority::GetCurPriority

/****************************************************************************
*                    CProcessPriority::GetCurPriorityName
* Result: CString
*       String name
****************************************************************************/

CString CProcessPriority::GetCurPriorityName()
    {
     LPPROCESS_PRIORITY_INFO pinfo = (LPPROCESS_PRIORITY_INFO)GetCurItem();     
     CString s;
     s.LoadString(pinfo->id);
     return s;
    } // CProcessPriority::GetCurPriorityName

/****************************************************************************
*                          CProcessPriority::Select
* Inputs:
*       DWORD priority: priority value to select
* Result: BOOL
*       TRUE if successful
*       FALSE if cannot be selected
* Effect: 
*       Selects the desired priority
****************************************************************************/

BOOL CProcessPriority::Select(DWORD priority)
    {
     for(int i = 0; i < GetCount(); i++)
        { /* search */
         LPPROCESS_PRIORITY_INFO info = (LPPROCESS_PRIORITY_INFO)GetItemDataPtr(i);
         if(info->priority == priority)
            { /* found it */
             SetCurSel(i);
             return TRUE;
            } /* found it */
        } /* search */
     // Note this critically depends on the fact that the control is actually initialized
     // and therefore this recursion will not be infinite
     Select(NORMAL_PRIORITY_CLASS);
     return FALSE;
    } // CProcessPriority::Select

/****************************************************************************
*                   CProcessPriority::GetCurThreadPriority
* Inputs:
*       int i: Index of process priority item
* Result: int
*       desired thread priority for that priority index
****************************************************************************/

int CProcessPriority::GetCurThreadPriority(int i)
    {
     const LPPROCESS_PRIORITY_INFO info = (LPPROCESS_PRIORITY_INFO)GetCurItem();     
     return info->threadPriority[i];
    } // CProcessPriority::GetCurThreadPriority
